import json
import requests

def analysis(password, path, title):

    from pdfminer.pdfinterp import PDFResourceManager, PDFPageInterpreter
    from pdfminer.converter import TextConverter
    from pdfminer.layout import LAParams
    from pdfminer.pdfpage import PDFPage
    from io import StringIO

    print('Convering pdf to text ...')
    rsrcmgr = PDFResourceManager()
    retstr = StringIO()
    codec = 'utf-8'
    laparams = LAParams()
    device = TextConverter(rsrcmgr, retstr, codec=codec, laparams=laparams)
    fp = open(path, 'rb')
    interpreter = PDFPageInterpreter(rsrcmgr, device)
    password_pdf = ""
    maxpages = 0
    caching = True
    pagenos=set()

    for page in PDFPage.get_pages(fp, pagenos, maxpages=maxpages, password=password_pdf, caching=caching, check_extractable=True):
        interpreter.process_page(page)

    text = retstr.getvalue()
    full_text = text.replace('-\n', '').replace('’', "'")
    fp.close()
    device.close()
    retstr.close()
    text = text.replace('-\n', '').replace('’', "'")
    lines = text.split('\n')
    lines_section_ids_dict = {}
    lines_section_ids = []
    for i, line in enumerate(lines[1:-2]):
        if len(lines[i-1]) == 0 and len(lines[i+1]) == 0 and len(lines[i]) > 3 and not str(lines[i]).isdigit():
            lines_section_ids_dict[i] = lines[i]
            lines_section_ids.append(i)

    ref_id = -1
    data = []
    for id in lines_section_ids_dict:
        data.append((lines_section_ids_dict[id], id))
    data = dict(data)

    final_data = {}
    final_data['paper_title'] = title
    final_data['full_text'] = full_text
    try:
        ref_id = data['References']
    except KeyError:
        ref_id = len(lines) - 1
    for i, id in enumerate(lines_section_ids):
        if i < len(lines_section_ids) - 1 and id < ref_id:
            start = lines_section_ids[i]
            end = lines_section_ids[i+1]
            interval_lines = lines[start+1:end]
            interval_lines_txt = ' '.join(interval_lines)
            if interval_lines and len(interval_lines_txt) > 100:
                final_data[lines_section_ids_dict[start]] = ' '.join(interval_lines)
                
    print('Uploading text ...')
    response = requests.post('http://tzagerlib1-env.eba-wjp8tqpj.eu-west-2.elasticbeanstalk.com/paper_analysis/' + password, json=json.dumps(final_data))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data





import networkx as nx
import matplotlib.pyplot as plt
import pickle
import pkg_resources


def load_example_networks(name='bayesian_network_sm1', save_network_image=False):
    location = pkg_resources.resource_filename('tzager', 'data/' + name)
    network = pickle.load(open(location, 'rb'))
    if save_network_image:
        G = network['graph']
        nx.draw_networkx(G)
        plt.savefig('Bayesian_Network.png')
    return network

def create_network(variable_connections, probabilities, save_network_image=False):    
    G = nx.DiGraph()
    edges = []
    for node in variable_connections:
        for connected_node in variable_connections[node]:
            edges.append((node, connected_node))
    G.add_edges_from(edges)

    if not nx.is_directed_acyclic_graph(G):
        print("Invalid Network ...\n", "Give a directed acyclic graph (DAG) ...")
        return ''
    else:
        nodes = G.nodes()

        if save_network_image:
            nx.draw_networkx(G)
            plt.savefig('Bayesian_Network.png')

        return {'graph': G, 'probabilities': probabilities}

def full_joint_probability(assignment, network):
    assignment_pr = 1
    G = network['graph']
    CPT = network['probabilities']
    nodes = G.nodes
    for n in nodes:
        parents = list(G.predecessors(n))
        if type(CPT[n]) is list:
            case_of_assignment = {}
            for p in parents:
                case_of_assignment[p] = assignment[p]
            for case in CPT[n]:
                if case[0] == case_of_assignment:
                    if assignment[n] == 'T':
                        assignment_pr *= case[1]
                    else:
                        assignment_pr *= (1 - case[1])
                    break
        else:
            assignment_pr *= CPT[n][assignment[n]]
    return {'Pr': assignment_pr}

def inference(query, network, algorithm='enumeration'):

    from itertools import product
    possible_values = ["T", "F"]
    G = network['graph']
    nodes = G.nodes
    query_variables = query['variable']
    evidence_variables = query['given']
    total_probability = 0
    if algorithm == 'enumeration':
        hidden_variables = []
        for n in nodes:
            if n not in query_variables and n not in evidence_variables:
                hidden_variables.append(n)
        print('Query Variables:', list(query_variables.keys()))
        print('Evidence Variables:', list(evidence_variables.keys()))
        print('Hidden Variables:', hidden_variables)
        print()
    
        hidden_variables_dict = {}
        for i,h in enumerate(hidden_variables):
            hidden_variables_dict[i] = h

        assignment = {**query_variables, **evidence_variables}
        
        hidden_variables_possible_assignments = list(product(possible_values, repeat=len(hidden_variables)))
        for pair in hidden_variables_possible_assignments:
            for i, value in enumerate(pair):
                assignment[hidden_variables_dict[i]] = value
            total_probability += full_joint_probability(assignment, network)['Pr']
        return {'Inference Pr': total_probability}
    
    elif algorithm == 'elimination':
        pass
    
    elif algorithm == 'approximation':
        pass




import json
import requests


def get_data(password, concept_list, filters=[]):
    data_dict = {'concept_list': concept_list, 'filters': filters}

    response = requests.get('https://cloud.bolooba.com:25556/diseases_data/' + password, json=json.dumps(data_dict))
    if response.status_code == 200:
        data = dict(response.json())
        
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data



import json
import requests


def get_data(password, question, type_of, path='None'):
    if type_of == 'list':
        question = '|'.join(question)
    
    response = requests.get('https://cloud.bolooba.com:25556/get_why/' + password + '/' + question + '/' + type_of + '/' + path)
    if response.status_code == 200:
        data = dict(response.json())
        
    else:
        data = {'error': response.status_code}
        data = dict(data)
    return data


import json
import requests

def question(password, discussion_id, query, question_id=None):
    response = requests.post('http://tzagerlib1-env.eba-wjp8tqpj.eu-west-2.elasticbeanstalk.com/question/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id, 'question_id': question_id}))
    # response = requests.post('http://127.0.0.1:5000/question/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id}))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)

def simulate(password, discussion_id, query, question_id=None):
    response = requests.post('http://tzagerlib1-env.eba-wjp8tqpj.eu-west-2.elasticbeanstalk.com/simulate/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id, 'question_id': question_id}))
    # response = requests.post('http://127.0.0.1:5000/question/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id}))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)

def get_discussion_info(password, discussion_id):
    response = requests.post('http://tzagerlib1-env.eba-wjp8tqpj.eu-west-2.elasticbeanstalk.com/get_discussion_info/' + password, json=json.dumps({'discussion_id': discussion_id}))
    # response = requests.post('http://127.0.0.1:5000/question/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id}))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)

def dig_deeper(password, discussion_id, question_id, node):
    response = requests.post('http://tzagerlib1-env.eba-wjp8tqpj.eu-west-2.elasticbeanstalk.com/dig_deeper/' + password, json=json.dumps({'discussion_id': discussion_id, 'question_id': question_id, 'node': node}))
    # response = requests.post('http://127.0.0.1:5000/question/' + password, json=json.dumps({'query':query, 'discussion_id': discussion_id}))
    if response.status_code == 200:
        data = dict(response.json())
    else:
        data = {'error': response.status_code}
        data = dict(data)